#define	F_CPU 	1200000UL

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>

//PB.0>CLOCK PB.1>DATA PB.2>STROBE PB.3>SCL PB.4>SDA
#define	CLOCK	0
#define	DATA	1
#define	STROBE	2

#define	ON(x)	PORTB|=(1<<x)
#define	OFF(x)	PORTB&=~(1<<x)

#define	SCL		3
#define	SDA		4
#define	ACK		1
#define	NACK	0

#define	SCL_HIGH	DDRB&=~(1<<SCL)
#define	SCL_LOW		DDRB|=(1<<SCL)
#define	SDA_HIGH	DDRB&=~(1<<SDA)
#define	SDA_LOW		DDRB|=(1<<SDA)


unsigned char k, MINUTS, HOURS;

//0>E, 1>A, 2>F, 3>B, 4>G, 5>C, 6>Dp, 7>D
//A>1, B>3, C>5, D>7, E>0, F>2, G>4, Dp>6
unsigned char PROGMEM DIG[]=
{
	0b01010000,	//0
	0b11010111,	//1
	0b01100100,	//2
	0b01000101,	//3
	0b11000011,	//4
	0b01001001,	//5
	0b01001000,	//6
	0b11010001,	//7
	0b01000000,	//8
	0b01000001,	//9
	0b11111101,	//10 -     ( )
	0b11111011,	//11 -    
	0b11110111,	//12 -    
	0b11111110	//13 -     ( )
};

//	**********************************************
//	*      DS1307 **
//	********************************************
void i2c_start (void)
{
	SCL_HIGH;
	SDA_HIGH;
	SDA_LOW;
	SCL_LOW;
}

void i2c_stop (void)
{
	SDA_LOW;
	SCL_HIGH;
	SDA_HIGH;
}

void i2c_send_byte (unsigned char data)
{
	SCL_LOW;
	for(k=8;k>0;k--)
	{
		if(data&(1<<(k-1))) SDA_HIGH;
		else SDA_LOW;
		SCL_HIGH;
		SCL_LOW;
	}
	SCL_HIGH;
	SCL_LOW;
}

unsigned char i2c_receive_byte (unsigned char ACK_NACK)
{	
	unsigned char out_data=0;
	SCL_LOW;
	for(k=8;k>0;k--)
	{
		SCL_HIGH;
		if(PINB&(1<<SDA)) out_data|=(1<<(k-1));
		else out_data&=~(1<<(k-1));
		SCL_LOW;
	}
	if(ACK_NACK) SDA_LOW;
	else SDA_HIGH;
	SCL_HIGH;
	SCL_LOW;
	SDA_HIGH;

	return out_data;
}

void rtc_read (void)
{
	i2c_start();
	i2c_send_byte(0xD0);		//    0 = 
	i2c_send_byte(1);			//  ,    
	i2c_start();
	i2c_send_byte(0xD1);		//    1 = 
	MINUTS=i2c_receive_byte(ACK);	// 
	HOURS=i2c_receive_byte(NACK);	// 
	i2c_stop();
}
//	********************************************
//	*      DS1307 **
//	**********************************************


//	**********************************************
//	*        **
//	********************************************
void load_to_register (unsigned char data)
{
	if(data) ON(DATA);
	else OFF(DATA);
	ON(CLOCK);
	OFF(CLOCK);
	OFF(DATA);
}

void send_to_ind(unsigned char ind_num, unsigned char ind_data)
{	
	for(k=8;k>0;k--)
	{
		if(pgm_read_byte(DIG+ind_num)&(1<<(k-1))) load_to_register(1);
		else load_to_register(0);
	}

	for(k=8;k>0;k--)
	{
		if((k==7) && (ind_num==11)) load_to_register(0);
		else
		{
			if(pgm_read_byte(DIG+ind_data)&(1<<(k-1))) load_to_register(1);
			else load_to_register(0);
		}
	}

	ON(STROBE);
	OFF(STROBE);
}

void show_time (void)
{
	send_to_ind(10, HOURS>>4);				//      
	_delay_us(900);
	send_to_ind(11, HOURS&0b00001111);		//      
	_delay_us(900);
	send_to_ind(12, MINUTS>>4);				//      
	_delay_us(900);
	send_to_ind(13, MINUTS&0b00001111);		//      
}
//	********************************************
//	*        **
//	**********************************************


//	*****************************
//	*     **
//	***************************
ISR (_VECTOR(2))
{
	//    
	send_to_ind(13, MINUTS&0b00001111);
	_delay_ms(100);
	if(PINB&(1<<5))	goto end;

	_delay_ms(1000);
	while(!(PINB&(1<<5)))
	{
		MINUTS++;
		if((MINUTS&0b00001111)>0b00001001) MINUTS&=0b11110000;
		send_to_ind(13, MINUTS&0b00001111);
		_delay_ms(1000);
	}
	
	//    
	send_to_ind(12, MINUTS>>4);
	while(PINB&(1<<5));
	_delay_ms(1000);

	while(!(PINB&(1<<5)))
	{
		MINUTS=MINUTS+0x10;
		if((MINUTS>>4)>0b00000101) MINUTS&=0b00001111;
		send_to_ind(12, MINUTS>>4);
		_delay_ms(1000);
	}

	//    
	send_to_ind(11, HOURS&0b00001111);
	while(PINB&(1<<5));
	_delay_ms(1000);
	while(!(PINB&(1<<5)))
	{
		HOURS++;
		if((HOURS&0b00001111)>0b00001001) HOURS&=0;
		send_to_ind(11, HOURS&0b00001111);
		_delay_ms(1000);
	}

	//    
	send_to_ind(10, HOURS>>4);
	while(PINB&(1<<5));
	_delay_ms(1000);
	while(!(PINB&(1<<5)))
	{
		HOURS=HOURS+0x10;
		if((HOURS>>4)>0b00000010) HOURS&=0b00001111;
		send_to_ind(10, HOURS>>4);
		_delay_ms(1000);
	}

	//  
	i2c_start();
	i2c_send_byte(0xD0);		//    0 = 
	i2c_send_byte(1);			//  ,    
	i2c_send_byte(MINUTS);		//   
	i2c_send_byte(HOURS);		// 
	i2c_stop();

end:	asm("nop");
}
//	***************************
//	*     **
//	*****************************


int main (void)
{
	//PB.0>CLOCK PB.1>DATA PB.2>STROBE PB.3>SCL PB.4>SDA PB.5>*
	PORTB	=	0b00100000;
	DDRB	=	0b11000111;

	GIMSK=(1<<PCIE);	//Pin Change Interrupt Enable
	PCMSK=(1<<PCINT5);	//Interrupt by Pin Change_5 Enable

	//  ( 0  7)
	i2c_start();
	i2c_send_byte(0xD0);		//    0 = 
	i2c_send_byte(0);			//  ,    
	i2c_send_byte(0);			//0  CH7  
	i2c_stop();

	asm("sei");

	while(1)
	{
		rtc_read();
		show_time();
	}

return 0;
}
